@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# ex: set sts=4 ts=4 sw=4 noet:
+# ex: set sts=4 ts=4 sw=4 et:
 # ## ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 #
 #   See COPYING file distributed along with the datalad package for the
@@ -78,8 +78,8 @@ def test_invalid_call(origin, tdir):
 
     # unavailable subdataset
     dummy_sub = ds.create('sub')
-    dummy_sub.uninstall()
-    assert_in('sub', ds.subdatasets(fulfilled=False, result_xfm='relpaths'))
+    dummy_sub.drop(what='all', reckless='kill', recursive=True)
+    assert_in('sub', ds.subdatasets(state='absent', result_xfm='relpaths'))
     # now an explicit call to publish the unavailable subdataset
     assert_raises(ValueError, ds.push, 'sub')
 
@@ -110,10 +110,10 @@ def mk_push_target(ds, name, path, annex=True, bare=True):
                 # for managed branches we need more fireworks->below
                 target.config.set(
                     'receive.denyCurrentBranch', 'updateInstead',
-                    where='local')
+                    scope='local')
     else:
         target = GitRepo(path=path, bare=bare, create=True)
-    ds.siblings('add', name=name, url=path, result_renderer=None)
+    ds.siblings('add', name=name, url=path, result_renderer='disabled')
     if annex and not bare and target.is_managed_branch():
         # maximum complication
         # the target repo already has a commit that is unrelated
@@ -165,9 +165,9 @@ def check_push(annex, src_path, dst_path):
 
     # configure a default merge/upstream target
     src.config.set('branch.{}.remote'.format(DEFAULT_BRANCH),
-                   'target', where='local')
+                   'target', scope='local')
     src.config.set('branch.{}.merge'.format(DEFAULT_BRANCH),
-                   DEFAULT_BRANCH, where='local')
+                   DEFAULT_BRANCH, scope='local')
 
     # don't fail when doing it again, no explicit target specification
     # needed anymore
@@ -511,7 +511,7 @@ def test_force_checkdatapresent(srcpath, dstpath):
                       message='Slated for transport, but no content present')
 
 
-@skip_if_on_windows  # https://github.com/datalad/datalad/issues/4278
+@known_failure_githubci_win
 @with_tempfile(mkdir=True)
 @with_tree(tree={'ria-layout-version': '1\n'})
 def test_ria_push(srcpath, dstpath):
@@ -525,7 +525,7 @@ def test_ria_push(srcpath, dstpath):
         'ok',
         src.create_sibling_ria(
             "ria+{}".format(get_local_file_url(dstpath, compatibility='git')),
-            "datastore"))
+            "datastore", new_store_ok=True))
     res = src.push(to='datastore')
     assert_in_results(
         res, action='publish', target='datastore', status='ok',
@@ -594,7 +594,7 @@ def test_gh1763(src, target1, target2):
     target1 = mk_push_target(src, 'target1', target1, bare=False)
     target2 = mk_push_target(src, 'target2', target2, bare=False)
     src.siblings('configure', name='target2', publish_depends='target1',
-                 result_renderer=None)
+                 result_renderer='disabled')
     # a file to annex
     (src.pathobj / 'probe1').write_text('probe1')
     src.save('probe1', to_git=False)
@@ -699,7 +699,7 @@ def test_auto_data_transfer(path):
     ds_a.save()
 
     # Should be the default, but just in case.
-    ds_a.repo.config.set("annex.numcopies", "1", where="local")
+    ds_a.repo.config.set("annex.numcopies", "1", scope="local")
     ds_a.create_sibling(str(path / "b"), name="b")
 
     # With numcopies=1, no data is copied with data="auto".
@@ -711,7 +711,7 @@ def test_auto_data_transfer(path):
     assert_not_in_results(res, action="copy")
 
     # numcopies=2 changes that.
-    ds_a.repo.config.set("annex.numcopies", "2", where="local")
+    ds_a.repo.config.set("annex.numcopies", "2", scope="local")
     res = ds_a.push(to="b", data="auto", since=None)
     assert_in_results(
         res, action="copy", target="b", status="ok",
@@ -732,7 +732,7 @@ def test_auto_data_transfer(path):
         path=str(ds_a.pathobj / "baz.dat"))
 
     # --auto also considers preferred content.
-    ds_a.repo.config.unset("annex.numcopies", where="local")
+    ds_a.repo.config.unset("annex.numcopies", scope="local")
     ds_a.repo.set_preferred_content("wanted", "nothing", remote="b")
     res = ds_a.push(to="b", data="auto", since=None)
     assert_not_in_results(
@@ -859,7 +859,7 @@ def test_push_git_annex_branch_many_paths_same_data(path):
 def test_push_matching(path):
     path = Path(path)
     ds = Dataset(path / "ds").create(force=True)
-    ds.config.set('push.default', 'matching', where='local')
+    ds.config.set('push.default', 'matching', scope='local')
     ds.save()
     remote_ds = mk_push_target(ds, 'local', str(path / 'dssibling'),
                                annex=True, bare=False)
@@ -929,7 +929,7 @@ def test_nested_pushclone_cycle_allplatforms(origpath, storepath, clonepath):
     store_url = 'ria+' + get_local_file_url(storepath)
     with chpwd(orig_super.path):
         run(['datalad', 'create-sibling-ria', '--recursive',
-             '-s', 'store', store_url])
+             '-s', 'store', store_url, '--new-store-ok'])
         run(['datalad', 'push', '--recursive', '--to', 'store'])
 
     # we are using the 'store' sibling's URL, which should be a plain path
@@ -986,8 +986,8 @@ def test_push_custom_summary(path):
 
     # These options are true by default and our tests usually run with a
     # temporary home, but set them to be sure.
-    ds.config.set("advice.pushUpdateRejected", "true", where="local")
-    ds.config.set("advice.pushFetchFirst", "true", where="local")
+    ds.config.set("advice.pushUpdateRejected", "true", scope="local")
+    ds.config.set("advice.pushFetchFirst", "true", scope="local")
     with swallow_outputs() as cmo:
         ds.push(to="sib", result_renderer="default", on_failure="ignore")
         assert_in("Hints:", cmo.out)
